<?php

class Entry
{
    private $db;
    private $columns = [];
    private $table = "";
    public function __construct($db = null, $table = "")
    {
        if ($db == null) {
            $this->db = new db('pdo');
        } else {
            $this->db = $db;
        }
        $this->table = $table;
        $this->columns = $this->db->getColumnNames($this->table);
        foreach ($this->columns as $key => $value) {
            $this->columns[$value] = $value;
        }
        return $this;
    }
    public function getColumnNames() {
        return $this->columns;
    }
    public function getLastQuery()
    {
        return $this->db->getLastQuery();
    }
    public function setLog($log)
    {
        $this->db->log = $log;
    }
    public function setExecute($exec)
    {
        $this->db->execute = $exec;
    }
    public function getTable()
    {
        return $this->table;
    }
    public function setTable($table = "")
    {
        $this->table = $table;
    }
    public function getColNames($num = null)
    {
        return $num != null ? $this->columns[$num] : $this->columns;
    }
    public function getAllAtLvl($fields = "*", $lvl = 0, $parent = "rodzic")
    {
        return $this->db->select($this->table, $fields, NULL, "WHERE `{$parent}` = {$lvl}");
    }
    public function getAll($fields = "*", $sort = "")
    {
        return $this->db->select($this->table, $fields, NULL, $sort);
    }
    public function getAllWhere($fields = "*", $where = "")
    {
        return $this->db->select($this->table, $fields, null, $where != "" ? "WHERE {$where}" : null);
    }
    public function getAtLvl($id = null, $fields = "*", $lvl = 0, $parent = "rodzic")
    {
        $val = $this->db->select($this->table, $fields, null, "WHERE `{$parent}` = {$lvl}" . $id != null ? " AND `id` = {$id}" : null);
        if (isset($id)) {
            $val = $val[0];
        }

        return $val;
    }
    public function get($id = null, $fields = "*")
    {
        $val = $this->db->select($this->table, $fields, null, $id != null ? "WHERE `id` = {$id}" : null);
        if (isset($id)) {
            $val = $val[0];
        }

        return $val;
    }
    public function getWhere($id = null, $fields = "*", $where = "")
    {
        echo ($id != null || $where != "" ? "WHERE" : "") . ($id != null ? " `id` = {$id}" : "") . ($id != null && $where != "" ? " AND" : "") . ($where != "" ? " {$where}" : "");
        $val = $this->db->select($this->table, $fields, null, ($id != null || $where != "" ? "WHERE" : "") . ($id != null ? " `id` = {$id}" : "") . ($id != null && $where != "" ? " AND" : "") . ($where != "" ? " {$where}" : ""));
        if (isset($id)) {
            $val = $val[0];
        }

        return $val;
    }
    public function save($fields, $values = null)
    {
        $this->setHighestID();
        $this->db->insert($this->table, null, $fields);
        return $this;
    }
    public function saveID($id = null, $fields, $values = null)
    {
        if ($id != null) {
            $this->db->update($this->table, "WHERE `id` = {$id}", $fields);
        } else {
            $this->save($fields);
        }
        return $this;
    }
    public function swap($valFrom, $valTo, $swapCol, $fields) {
        $this->db->swap($this->table, $valFrom, $valTo, $swapCol, $fields);
    }
    public function delete($id)
    {
        $this->db->delete($this->table, "WHERE `id` = {$id}");
        $this->setHighestID();
        return $this;
    }
    private function setHighestID()
    {
        $this->db->query("ALTER TABLE `" . $this->table . "` AUTO_INCREMENT " . ($this->maxID() + 1));
    }
    public function maxID()
    {
        return $this->db->select($this->table, "max(`id`) AS `max`")[0]['max'];
    }
}

class Module
{

    public $defaultLang = "";
    public $langs = [];
    private $entries = INF;
    private $table;
    private $tables = [];

    public function __construct($db = null, $langs = "", $table = "", $ref = 0)
    {
        if ($db == null) {
            $this->db = new db('pdo');
        } else {
            $this->db = $db;
        }

        $this->table = $table;
        $this->ref = $ref;
        if (gettype($langs) == "string") {
            $langs = explode(",", str_replace(", ", ",", $langs));
        }

        $this->langs = $langs;
        $this->defaultLang = strtolower($langs[0]);
        foreach ($langs as $key) {
            if (strtolower($key) == $this->defaultLang) {
                $lang = "";
            } else {
                $lang = $key;
            }

            //$langTable = $table . ($lang != "" ? "_" . $lang : "");
            $langTable = "{$table}{$lang}";
            $this->tables[strtolower($key)] = new Entry($db, $langTable);
            if (count($this->tables[strtolower($key)]->getColNames()) <= 0) {
                $db->copyTables($table, $langTable, true);
            }
            $entries = $this->db->select($langTable, "COUNT(*) AS `count`");
            if (isset($entries[0]) && isset($entries[0]['count']) && $entries[0]['count'] < $this->entries) {
                $this->tables[strtolower($key)]->entries = $entries[0]['count'];
                $this->entries = $entries[0]['count'];
            }
        }
        return $this;
    }

    public function log($lang = null, $log = false)
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        $this->tables[$lang]->setLog($log);
        return $this;
    }

    public function logAllLang($log = false)
    {
        foreach ($this->tables as $key => $value) {
            $this->tables[$key]->setLog($log);
        }
        return $this;
    }

    public function getTableName() {
        return $this->table;
    }

    public function getColumnNames() {
        return $this->tables[$this->defaultLang]->getColumnNames();
    }

    public function get($id = null, $fields = null, $lang = null)
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        return $this->tables[$lang]->get($id, $fields);
    }

    public function getEntriesCount($lang = null) {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        return $this->tables[$lang]->entries;
    }

    public function getAllLang($id = null, $fields = null)
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        $tmp = [];
        foreach ($this->tables as $key => $value) {
            $tmp[$key] = $this->tables[$key]->get($id, $fields);
        }
        return $tmp;
    }

    public function getWhere($id = null, $fields = null, $lang = null, $where = "")
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        return $this->tables[$lang]->getWhere($id, $fields, $where);
    }

    public function getAllLangWhere($id = null, $fields = null, $where = "")
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        $tmp = [];
        foreach ($this->tables as $key => $value) {
            $tmp[$key] = $this->tables[$key]->getWhere($id, $fields, $where);
        }
        return $tmp;
    }

    public function listAll($lang = null, $fields = null, $sort = "")
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        return $this->tables[$lang]->getAll($fields, $sort);
    }

    public function listAllLang($fields = null)
    {
        $tmp = [];
        foreach ($this->tables as $key => $value) {
            $tmp[$key] = $this->tables[$key]->getAll($fields);
        }
        return $tmp;
    }

    public function listAllAtLvl($lang = null, $fields = null, $lvl = 0, $parent = "rodzic")
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        return $this->tables[$lang]->getAllAtLvl($fields, $lvl, $parent);
    }

    public function listAllLangAtLvl($fields = null, $lvl = 0, $parent = "rodzic")
    {
        $tmp = [];
        foreach ($this->tables as $key => $value) {
            $tmp[$key] = $this->tables[$key]->getAllAtLvl($fields, $lvl, $parent);
        }
        return $tmp;
    }

    public function add($fields = null, $lang = null, $id = null)
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        /*
        foreach ($_POST as $key => $value) {
        switch ($POST_priority) {
        case "update":
        case 1:
        if (array_key_exists($key, $_POST)) {
        $fields[$key] == $_POST[$key];
        }
        break;
        case "insert":
        case 2:
        $fields[$key] = $_POST[$key];
        break;
        default:
        break;
        }
        }
         */
        $this->tables[$lang]->saveID($id, $fields);
        echo $this->tables[$lang]->getLastQuery();
    }

    public function addAllLang($fields = null, $id = null)
    {
        foreach ($this->tables as $key => $value) {
            $this->add($fields, $key, $id);
        }
        return $this;
    }

    public function update($id = null, $fields = null, $lang = null)
    {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        if ($id != null) {
            $this->tables[$lang]->saveID($id, $fields);
        } else {
            trigger_error("No ID", E_USER_ERROR);
        }

        return $this;
    }

    public function updateAllLang($id = null, $fields = null, $POST_priority = false)
    {
        if ($id != null) {
            foreach ($this->tables as $key => $value) {
                $this->tables[$key]->saveID($id, $fields);
            }
        } else {
            trigger_error("No ID", E_USER_ERROR);
        }

        return $this;
    }

    public function delete($id = null, $lang = null)
    {
        if ($id == null) {
            return $this;
        }

        if ($lang == null) {
            $lang = $this->defaultLang;
        }

        $this->tables[$lang]->delete($id);
    }
    public function deleteAllLang($id = null)
    {
        if ($id == null) {
            return $this;
        }

        foreach ($this->tables as $item) {
            $item->delete($id);
        }
        return $this;
    }

    public function swap($valFrom, $valTo, $swapCol, $fields = "*", $lang) {
        if ($lang == null) {
            $lang = $this->defaultLang;
        }
        $this->tables[$lang]->swap($valFrom, $valTo, $swapCol, $fields);
    }

    public function swapAllLang($valFrom, $valTo, $swapCol, $fields = "*") {
        foreach ($this->tables as $key => $value) {
            $this->tables[$key]->swap($valFrom, $valTo, $swapCol, $fields);
        }
    }

    public function copyNewLang($langFrom = "", $langTo = "", $ignore = false)
    {
        if (gettype($langTo) == "string") {
            $langTo = explode(",", $langTo);
        }

        $tableLangFrom = $this->table . ($langFrom != "" && $langFrom != $this->defaultLang ? "_" . $langFrom : "");
        foreach ($langTo as $item) {
            $tableLangTo = $this->table . ($item != "" && $item != $this->defaultLang ? "_" . $item : "");
            $this->db->copy($tableLangFrom, $tableLangTo);
        }
        return $this;
    }

}

class Panel
{
    /*
    $displayTable:

    ["database_table_column_1" => [
    "name" => "Field name" / function(data),
    "styles" => "display: none" / function(data),
    "value" => NULL / "Value" / function(data) // IF NOT SPECIFIED OR NULL/EMPTY THEN DATABASE KEY VALUE,
    "valueStyles" => "display: none" / function(data)
    ] / function(data),
    "options" => "options", // WHERE "options" ARE LISTED SEPARATED BY SPACE, SUPPORTED OPTIONS:
                            // enableLeveling - enable nesting of items
                            // enableGrouping - allow to group and list only group of items
    "level" => 0,           // WHEN "enableLeveling" enabled, display items of n-level
    "parent" = "rodzic"]    // WHEN "enableLeveling" enabled, column name of parent

    // Argument "data" contains:
    "table" -> current working table, class Module or any similar to table
    contains $db, which is used to connect and query database
    to correct use:
    function($data) {
    return $data['db']->query("query");
    }
    "row"   -> current processed row
    "key"   -> current processed column (key, name)
    "value" -> current processed column (value)
     */

    private $table;
    public $langs = [];
    public $defaultLang = "";
    // LISTED - DATA FROM OTHER TABLE PRESENTED IN <SELECT> LIST
    private $flags = ["table" => ["leveled" => false, "grouped" => false, "listed" => false], "editor" => ["leveled" => false, "grouped" => false, "listed" => false]];
    public function __construct($table = null, $langs = "", $k = 3)
    {
        $this->table = $table;
        if ($langs == null || $langs == "") {
            $langs = $table->langs ? $table->langs : $_SESSION['lang'];
        }

        if (gettype($langs) == "string") {
            $langs = explode(",", str_replace(", ", ",", $langs));
        }

        $this->defaultLang = strtolower($langs[0]);
        $this->langs = $langs;
        $this->k = $k;
    }

    /*
    $settings:

    [
    "id" => 1,                     // Current element ID, which is
    used to determine current level
    "parentColumnName" => "rodzic" // Parent column name, which is
    used to determine parent ID
    ]
     */

    public static function renderTree($groupData = [], $options = []) {
        $groupSet = [];
        // (isset(...) || array_key_exists(search, in_array))
        // SORTING BY IDs
        foreach ($groupData as $gKey => $gValue) {
            $groupSet[$gValue[$options['groupIDColumn']]] = $gValue;
        }
        // REMOVING ORPHANS
        foreach ($groupSet as $gKey => $gValue) {
            $par = $gValue[$options['groupParentColumn']];
            if (!(isset($groupSet[$par]) || array_key_exists($par, $groupSet)) && $par > 0) {
                unset($groupSet[$gKey]);
            }
        }
        // REMOVING SELF-POINTINGS
        foreach ($groupSet as $gKey => $gValue) {
            if ($gValue[$options['groupIDColumn']] == $gValue[$options['groupParentColumn']]) unset($groupSet[$gKey]);
        }
        // GETTING LEVEL (HOW DEEP)
        $groupMaxLevel = 0;
        foreach ($groupSet as $gKey => $gValue) {
            $groupLevel = -1;
            do {
                $testParent = $gValue[$options['groupParentColumn']];
                $gValue = $groupSet[$testParent];
                $groupLevel++;
            } while ($testParent > 0);
            $groupSet[$gKey]['level'] = $groupLevel;
            if ($groupLevel > $groupMaxLevel) $groupMaxLevel = $groupLevel;
        }
        $groupMap = $groupSet;
        $groupMapIDs = "";
        $groupLevel = 0;
        //$groupMapIDs = implode(",", array_keys($groupSet));
        foreach ($groupMap as $gKey => $gValue) {
            if ($gValue['level'] == $groupLevel) {
                $groupMapIDs .= $gKey . ",";
                unset($groupMap[$gKey]);
            }
        }
        $groupMap = array_reverse($groupMap);
        do {
            $groupLevel++;
            foreach ($groupMap as $gKey => $gValue) {
                if ($gValue['level'] == $groupLevel) {
                    $groupMapIDs = preg_replace("/(?<=,|^)" . ($gValue[$options['groupParentColumn']]) . ",/", ($gValue[$options['groupParentColumn']]) . ",{$gValue[$options['groupIDColumn']]},", $groupMapIDs);
                    unset($groupMap[$gKey]);
                }
            }
        } while ($groupLevel <= $groupMaxLevel);
        $groupMapIDs = preg_replace("/(?:,{2,})/", ",", $groupMapIDs);
        $groupMapIDs = preg_replace("/(?:^,)|(?:,$)/", "", $groupMapIDs);
        $groupMapIDs = array_slice(explode(",", $groupMapIDs), 0, count($groupSet));
        $groupMap = [];
        foreach ($groupMapIDs as $value) {
            $displayTxt = str_repeat($options['groupIndentStr'], $groupSet[$value]['level']) . ($groupSet[$value]['level'] > 0 ? $options['groupIndentSeparator'] : "") . $groupSet[$value][$options['groupDisplayTextColumn']];
            $val = ["id" => $value, "txt" => $displayTxt];
            array_push($groupMap, $val);
        }
        return $groupMap;
    }

    public function renderLevel($settings = [])
    {

    }

    public function renderTable($displayTable = [], $renderData = [], $amount = [], $sort = "")
    {
        $options = ['options' => ""];
        $addButton = "<div id=\"menu2\"><a href=\"content.php?k=" . $this->table->ref . "{level}&o=1\">dodaj{addDisplayText}</a></div>";
        $groupColumn = -1;

        /* --- HEADER OF TABLE --- */

        $table .= "<table id=\"TABELA\" class=\"list\"><thead>
        <tr style=\"height: 50px; background: #f5f5f5;\">%DRAGh%";
        if (is_callable($displayTable, false)) {
            //$table .= $displayTable("header", $this->table);
        } else {
            $gI = 0;
            foreach ($displayTable as $key => $value) {
                if (is_callable($value, false)) {
                    $table .= $value(["db" => $this->table->db, "table" => $this->table, "row" => $rowValue, "key" => $entryKey, "value" => $entryValue], $renderData);
                } else {
                    if (gettype($value) != "array") {
                        $options[$key] = $value;
                        unset($displayTable[$key]);
                        continue;
                    }
                    $table .= "<th";
                    if (isset($value['styles']) && is_callable($value['styles'])) {
                        $table .= " style=\"" . $value['styles'](["db" => $this->table->db, "table" => $this->table, "row" => $rowValue, "key" => $entryKey, "value" => $entryValue], $renderData) . "\"";
                    } else {
                        $table .= (isset($value['styles']) ? " style=\"" . $value['styles'] . "\"" : "");
                    }
                    if (isset($value['multiValues']) && isset($value['multiValuesDisplayTextSeparator'])) {
                        $table .= " data-values-separator=\"" . $value['multiValuesDisplayTextSeparator'] . "\"";
                        $groupColumn = $gI;
                    }
                    $table .= ">";
                    if (isset($value['name']) && is_callable($value['name'])) {
                        $table .= $value['name'](["db" => $this->table->db, "table" => $this->table, "row" => $rowValue, "key" => $entryKey, "value" => $entryValue], $renderData);
                    } else {
                        $table .= isset($value['name']) ? $value['name'] : $key;
                    }

                    $table .= "</th>";
                }
                $gI++;
            }
            if (isset($options['disableButtons']))
                if (stripos($options['disableButtons'], "edit") !== false)
                    if (stripos($options['disableButtons'], "delete") !== false) $options['disableActions'] = true;
            
            if (!isset($options['disableActions']) || $options['disableActions'] != true) $table .= "<th style=\"width: 20%; text-align: center;\"> <b>Akcje</b></th>";
        }
        $table .= "</tr></thead><tbody>";

        /* --- CHECKING OPTIONS --- */

        if ($options['options']) $options['options'] = ' ' . $options['options'];

        // LEVELING / NESTING
        $breadcrumbs = "";

        if (stripos($options['options'], "enableLeveling")) {
            if (!isset($options['levelIDColumn'])) $options['levelIDColumn'] = "id";
            if (!isset($options['levelParentColumn'])) $options['levelParentColumn'] = "rodzic";
            if (!isset($options['levelDisplayTextColumn'])) $options['levelDisplayTextColumn'] = "nazwa";
            if (!isset($options['levelSeparatorStr'])) $options['levelSeparatorStr'] = "::";

            $test = ["levelIDColumn", "levelParentColumn", "levelDisplayTextColumn"];
            $testCols = $this->table->getColumnNames();
            foreach ($test as $value) {
                if (!in_array($options[$value], $testCols)) { // COLUMN NOT FOUND, SO DISABLE LEVELING
                    $options['options'] = preg_replace("/enableLeveling/i", "", $options['options']);
                    break;
                }
            }
        }

        // GROUPING
        $grouping = "";
        $groupingJS = "";

        if (stripos($options['options'], "enableGrouping")) {
            if (!isset($options['groupIDColumn'])) $options['groupIDColumn'] = "id";
            if (!isset($options['groupParentColumn'])) $options['groupParentColumn'] = "rodzic";
            if (!isset($options['groupDisplayTextColumn'])) $options['groupDisplayTextColumn'] = "nazwa";
            if (!isset($options['groupIndentStr'])) $options['groupIndentStr'] = "-";
            if (!isset($options['groupIndentSeparator'])) $options['groupIndentSeparator'] = " ";

            if (!$this->table->db->tableExists($options['groupTable'])) {
                $options['options'] = preg_replace("/enableGrouping/i", "", $options['options']);
            }

            $test = ["groupIDColumn", "groupParentColumn", "groupDisplayTextColumn"];
            $testCols = $this->table->db->getColumnNames($options['groupTable']);
            foreach ($test as $value) {
                if (!in_array($options[$value], $testCols)) { // COLUMN NOT FOUND, SO DISABLE LEVELING
                    $options['options'] = preg_replace("/enableGrouping/i", "", $options['options']);
                    break;
                }
            }

        }

        if (stripos($options['options'], "enableGrouping")) {
            $this->flags["table"]["grouped"] = true;
            $grouping = "<div id=\"selectkateg\"><select id=\"showcat\" " . ($groupColumn > -1 ? "data-group-column=\"$groupColumn\"" : "") . " data-indent-str=\"" . $options['groupIndentStr'] . "\" data-indent-separator=\"" . $options['groupIndentSeparator'] . "\"><option selected=\"selected\" value=\"0\">wszystkie</option><option disabled=\"disabled\">---</option>";
            $groupData = $this->table->db->select($options['groupTable'], [$options['groupIDColumn'], $options['groupDisplayTextColumn'], $options['groupParentColumn']]);
            $groupMap = Panel::renderTree($groupData, $options);

            foreach ($groupMap as $gValue) {
                $grouping .= "<option value=\"{$gValue['id']}\">{$gValue['txt']}</option>";
            }
            $grouping .= "</select><label class=\"showcatlabel\">{$options['groupLabelText']}</label>";
        }

        // DataTables
        $dataTables = "";

        if (isset($options['paginator'])) {
            if (!isset($options['perPage'])) $options['perPage'] = isset($_GET['per_page']) ? $_GET['per_page'] : 10;
        }

        // TABLE WITH CONTENT
        $displayKeys = array_keys($displayTable);

        if (stripos($options['options'], "enableLeveling")) {
            $this->flags["table"]["leveled"] = true;
            $list = $this->table->listAllAtLvl(null, null, $options['level'], $options['levelParentColumn']);
            $parentID = $list[0][$options['levelParentColumn']];
            if ($list == NULL) $parentID = $options['level'];
            if ($options['level'] != 0 && $options['level'] != null && $parentID > 0) {
                do {
                    $list2[$parentID] = $this->table->get($parentID);
                    $parentID = $list2[$parentID][$options['levelParentColumn']];
                } while ($parentID > 0);
                ksort($list2);
                $breadcrumbs = "<p class=\"gingerbreads\"><a href=\"content.php?k=" . $this->table->ref . "\">MENU</a>";
                foreach ($list2 as $lKey => $lValue) {
                    $breadcrumbs .= "<span>" . $options['levelSeparatorStr'] . "</span>";
                    if ($lValue[$options['levelIDColumn']] == $options['level']) $breadcrumbs .= "<span>" . $lValue[$options['levelDisplayTextColumn']] . "</span>";
                    else $breadcrumbs .= "<a href=\"content.php?k=" . $this->table->ref . ($lValue[$options['levelIDColumn']] != 0 ? "&l=" . $lValue[$options['levelIDColumn']] : "") . "\">" . $lValue[$options['levelDisplayTextColumn']] . "</a>";
                }
                $breadcrumbs .= "</p>";
            }
        } else {
            $list = $this->table->listAll(null, null, $sort);
        }

        // GETTING TABLES WITH DATA
        $dataOld = [];
        foreach ($displayTable as $dKey => $dValue) {
            if (isset($dValue['valueFromTable']))
                if (!isset($dataOld[$dValue['valueFromTable']])) {
                    $dataOld[$dValue['valueFromTable']] = [$dValue['valueFromField'], $dValue['valueDisplayTextFromField']];
                    $dataOld[$dValue['valueFromTable']][2] = $this->table->db->select($dValue['valueFromTable'], (isset($dValue['valueFromField']) && isset($dValue['valueDisplayTextFromField']) ? [$dValue['valueFromField'], $dValue['valueDisplayTextFromField']] : "*"));
                }
        }
        $data = [];
        foreach ($dataOld as $dKey => $dValue) {
            $data[$dKey] = [];
            foreach ($dValue[2] as $key => $value) {
                $data[$dKey][$value[$dValue[0]]] = $value[$dValue[1]];
            }
        }
        unset($dataOld);

        foreach ($list as $rowKey => $rowValue) {
       
            $rowValue2 = [];
            foreach ($displayKeys as $dKey => $dVal) {
                $rowValue2[$dKey] = $rowValue[$dKey];
            }
            $table .= "<tr>%DRAGd%";
            foreach ($displayTable as $entryKey => $entryValue) {
                if (is_callable($entryValue, false)) {
                    $table .= $entryValue(["db" => $this->table->db, "table" => $this->table, "row" => $rowValue, "key" => $entryKey, "value" => $entryValue], $renderData);
                } else {
                    $table .= "<td";
                    if (isset($entryValue['valueStyles']) && is_callable($entryValue['valueStyles'])) {
                        $table .= " style=\"" . $entryValue['valueStyles'](["db" => $this->table->db, "table" => $this->table, "row" => $rowValue, "key" => $entryKey, "value" => $entryValue], $renderData) . "\"";
                    } else {
                        $table .= (isset($entryValue['valueStyles']) ? " style=\"" . $entryValue['valueStyles'] . "\"" : "");
                    }

                    $table .= ">";
                    if (isset($entryValue['value']) && is_callable($entryValue['value'])) {
                        $table .= $entryValue['value'](["db" => $this->table->db, "table" => $this->table, "row" => $rowValue, "key" => $entryKey, "value" => $entryValue], $renderData);
                    } else {
                        if (stripos($options['options'], "enableLeveling")) {
                            $table .= "<a href=\"content.php?k=" . $this->table->ref . ($rowValue[$options['levelIDColumn']] > 0 ? "&l=" . $rowValue[$options['levelIDColumn']] : "") . "\">";
                        }
                        $displayValue = isset($entryValue['value']) ? $entryValue['value'] : $rowValue[$entryKey];
                        if (isset($entryValue['multiValues'])) {
                            if ($entryValue['multiValues'] == "string") {
                                $table = substr($table, 0, -1) . "data-values=\"{$displayValue}\">";
                                $displayValue = preg_replace("/" .$entryValue['multiValuesSeparator'] . "{2,}/", $entryValue['multiValuesSeparator'], $displayValue);
                                $displayValue = preg_replace("/(?:^" . $entryValue['multiValuesSeparator'] . "+|" . $entryValue['multiValuesSeparator'] . "+$)/", "", $displayValue);
                                $displayValue = explode($entryValue['multiValuesSeparator'], $displayValue);
                            }
                        }
                        if (isset($entryValue['valueFromTable'])) {
                            $this->flags["table"]["listed"] = true;
                            foreach ($displayValue as $nKey => $nValue) {
                                if ($data[$entryValue['valueFromTable']][$nValue] != "") $displayValue[$nKey] = $data[$entryValue['valueFromTable']][$nValue];
                            }
                        }
                        if (gettype($displayValue) != "array") $displayValue = [$displayValue];
                        $displayValue = implode($entryValue['multiValuesDisplayTextSeparator'], $displayValue);
                        $table .= $displayValue;
                        if (stripos($options['options'], "enableLeveling")) {
                            $table .= "</a>";
                        }
                    }

                    $table .= "</td>";
                }
            }
            if (!isset($options['disableActions']) || $options['disableActions'] != true) {
                $actions = "<td>";
                if (!isset($options['disableButtons']) || stripos($options['disableButtons'], "edit") === false) $actions .= "<form id=\"" . $rowValue['id'] . "\" action=\"content.php?k={$this->k}&a=" . $rowValue['id'] . "&o=2\" method=\"POST\">
                    <input name=\"id\" type=\"hidden\" value=\"" . $rowValue['id'] . "\" />
                    <input class=\"edit\" type=\"submit\" name=\"edytuj\"  value=\"Edytuj\">
                </form>";
                if (!isset($options['disableButtons']) || stripos($options['disableButtons'], "delete") === false) $actions .= "<form id=\"del_" . $rowValue['id'] . "\" action=\"content.php?k={$this->k}&a=" . $rowValue['id'] . "&o=2\" method=\"POST\" onsubmit=\"return confirmDelete();\" />
                    <input name=\"delete\" type=\"hidden\" value=\"" . $rowValue['id'] . "\" />
                    <input class=\"delete\" type=\"submit\" name=\"usun\"  value=\"Usuń\">
                </form>";
                $actions .= "</td>";
            }
            if ($actions != "<td></td>") $table .= $actions;
            $table .= "</tr>";
        }

        $table .= "</tbody></table>";

        $table = preg_replace(["/%DRAG(d)%/", "/%DRAG(h)%/"], ((isset($options['dataTable']) && $options['dataTable'] == true) || isset($options['dataTables']) && $options['dataTables'] == true) ? ["<t$1 style=\"width: 25px\"><img src=\"images\moveupdown.svg\" style=\"height: 20px\" /></t$1>", "<t$1 style=\"width: 25px\"></t$1>"] : "", $table);

        if (isset($options['disableButtons']) && stripos($options['disableButtons'], "add") !== false) $addButton = ""; else $addButton = preg_replace_callback("/{([^$][a-zA-Z0-9-_]+)}/", function($rep) use ($options) {
            $txt = $options[$rep[1]];
            switch (mb_strtolower($rep[1])) {
                case "adddisplaytext":
                    $txt = " " . $txt;
                    break;
                case "level":
                    if (stripos($options['options'], "enableLeveling") && $options['level'] > 0) $txt = "&l=" . $txt;
                    else $txt = "";
            }
            return $txt;
        }, $addButton);
        $_SESSION['editorOptions'] = $options;
        echo $addButton . $breadcrumbs . $grouping . $table;
    }

    /*
    $editorFields:

    [
    "database_table_column_1" => [
    "lang"          => "all"/"single"/NULL/function(data)            // DEFAULT is NULL = ALL (one field for all languages)
    "displayName"   => "name"/function(data) // displayed field description
    "fieldName"     => "id"/function(data)
    "type"          => "short_text"/"long_text"/"html"/"file"/"date"/"time"/"dateTime"/"select"/function(data)   // DEFAULT is NULL = text
    "style"         => "display: none"/function(data)
    "value"         => "value"/function(data)
    "buttonText"    => "any_text"/function(data)
    "displayStyles" => "display: none"/function(data)
    "fieldStyles"   => "display: none"/function(data)
    "buttonStyles"  => "display: none"/function(data)
    ] / function(data)

    // If type is "select", then you need to provide ITEM LIST by
    additional "from" field, which contains:
    "from" => ["data" => entries from database, "name" => name of column containing names to display, "value" => name of column containing comparable values]

    // Argument "data" contains:
    "table" -> current working table, class Akt or any similar to table
    contains $db, which is used to connect and query database
    to correct use:
    function($data) {
    return $data['db']->query("query");
    }
    "key"   -> current processed column (key, name)
    "value" -> current processed column (value)
    ]
     */
    public function renderEditor($editorFields = [], $renderData = [], $id = null)
    {
        function type(&$txtVar, $lang, $key, $value, $data = "", &$fileCounter, &$post, $extData)
        {
            if (gettype($lang) != "string") {
                $lang = $lang['lang'];
            }
            if (isset($value['fieldName'])) {
                if (is_callable($value['fieldName'])) {
                    $fieldName = $value['fieldName'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
                } else {
                    $fieldName = $value['fieldName'];
                }

            } else {
                $fieldName = $key . (isset($lang) && $lang != null && $lang != "" ? "_{$lang}" : "");
            }
            $value['fieldName'] = $fieldName;
            $post[$fieldName] = ['column' => $key];
            if (isset($lang) && $lang != null && $lang != "") {
                $post[$fieldName]['lang'] = $lang;
            }

            switch (strtolower($value['type'])) {
                case "short_text":
                case "shorttext":
                case "text":
                    $data = htmlspecialchars($data);
                    $txtVar .= "<td><input type=\"text\" name=\"{$fieldName}\"";
                    if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) {
                        $txtVar .= " style=\"" . $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                    } else {
                        $txtVar .= (isset($value['fieldStyles']) ? " style=\"" . $value['fieldStyles'] . "\"" : "");
                    }
                    if (isset($value['attributes'])) {
                        if (is_callable($value['attributes'])) {
                            $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                            $value['attributes'] = $tmp;
                        } else {
                            if (gettype($value['attributes'] == "array")) {
                                $tmp = "";
                                foreach ($value['attributes'] as $aK => $aV) {
                                    if (gettype($aK) != "string") $aK = $aV;
                                    $tmp .= "{$aK}=\"{$aV}\" ";
                                }
                                $tmp = substr($tmp, 0, -1);
                                $value['attributes'] = $tmp;
                            }
                        }
                        $txtVar .= " " . $value['attributes'];
                    }
                    $txtVar .= ($data != "" ? "value=\"{$data}\"" : "") . "/></td>";
                    break;
                case "long_text":
                case "longtext":
                case "textarea":
                case "html":
                case "tinymce":
                    $tinymce = strtolower($value['type']) == "html" || strtolower($value['type']) == "tinymce" ? " class=\"yes\"" : "";
                    $txtVar .= "</tr><tr><td><textarea{$tinymce} name=\"{$fieldName}\"";
                    if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) {
                        $txtVar .= " style=\"" . $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                    } else {
                        $txtVar .= (isset($value['fieldStyles']) ? " style=\"" . $value['fieldStyles'] . "\"" : "");
                    }
                    if (isset($value['attributes'])) {
                        if (is_callable($value['attributes'])) {
                            $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                            $value['attributes'] = $tmp;
                        } else {
                            if (gettype($value['attributes'] == "array")) {
                                $tmp = "";
                                foreach ($value['attributes'] as $aK => $aV) $tmp .= "{$aK}=\"{$aV}\"";
                                $tmp = substr($tmp, 0, -1);
                                $value['attributes'] = $tmp;
                            }
                        }
                        $txtVar .= " " . $value['attributes'];
                    }

                    $txtVar .= ">{$data}</textarea>";
                    break;
                case "file":
                case "img":
                case "image":
                    $data = htmlspecialchars($data);
                    $txtVar .= "<td><input class=\"wpis\" type=\"text\" id=\"file_{$fileCounter}\" name=\"{$fieldName}\"";
                    if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) {
                        $txtVar .= " style=\"" . $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                    } else {
                        $txtVar .= (isset($value['fieldStyles']) ? " style=\"" . $value['fieldStyles'] . "\"" : "");
                    }
                    if (isset($value['attributes'])) {
                        if (is_callable($value['attributes'])) {
                            $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                            $value['attributes'] = $tmp;
                        } else {
                            if (gettype($value['attributes'] == "array")) {
                                $tmp = "";
                                foreach ($value['attributes'] as $aK => $aV) $tmp .= "{$aK}=\"{$aV}\"";
                                $tmp = substr($tmp, 0, -1);
                                $value['attributes'] = $tmp;
                            }
                        }
                        $txtVar .= " " . $value['attributes'];
                    }
                    $type = "";
                    switch (strtolower($value['type'])) {
                        case "img":
                        case "image":
                            $type = "type=1&";
                            break;
                    }
                    $txtVar .= ($data != "" ? "value=\"{$data}\"" : "") . "/><a href=\"filemanager/dialog.php?{$type}sort_by=name&descending=0&lang=pl&field_id=file_{$fileCounter}\" class=\"fancy\"";
                    if (isset($value['buttonStyles']) && is_callable($value['buttonStyles'])) {
                        $txtVar .= " style=\"" . $value['buttonStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                    } else {
                        $txtVar .= (isset($value['buttonStyles']) ? " style=\"" . $value['buttonStyles'] . "\"" : "");
                    }

                    $txtVar .= ">";
                    if (isset($value['buttonText']) && is_callable($value['buttonText'])) {
                        $txtVar .= $value['buttonText'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
                    } else {
                        $txtVar .= isset($value['buttonText']) ? $value['buttonText'] : "Wybierz";
                    }

                    $txtVar .= "</a></td>";
                    $fileCounter++;
                    break;
                case "select":
                case "list":
                    if ($value['from']) {
                        $txtVar .= "<td><select name=\"{$fieldName}\"";
                        if (isset($value['attributes'])) {
                            if (is_callable($value['attributes'])) {
                                $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                                $value['attributes'] = $tmp;
                            } else {
                                if (gettype($value['attributes'] == "array")) {
                                    $tmp = "";
                                    foreach ($value['attributes'] as $aK => $aV) $tmp .= "{$aK}=\"{$aV}\"";
                                    $tmp = substr($tmp, 0, -1);
                                    $value['attributes'] = $tmp;
                                }
                            }
                            $txtVar .= " " . $value['attributes'];
                        }
                        if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) {
                            $txtVar .= " style=\"" . $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                        } else {
                            $txtVar .= (isset($value['fieldStyles']) ? " style=\"" . $value['fieldStyles'] . "\"" : "") . ">";
                        }
                        if (isset($value['from']['firstEmpty']) && $value['from']['firstEmpty'] != false) $txtVar .= "<option value=\"\"" . ($sValue[$value['from']['value']] == $data ? " selected=\"selected\"" : "") . ">" . ($value['from']['firstEmptyDisplayText'] ? $value['from']['firstEmptyDisplayText'] : "...") . "</option>";
                        foreach ($value['from']['data'] as $sKey => $sValue) {
                            $txtVar .= "<option value=\"" . $sValue[$value['from']['value']] . "\"" . ($sValue[$value['from']['value']] == $data ? " selected=\"selected\"" : "") . ">" . $sValue[$value['from']['name']] . "</option>";
                        }
                        $txtVar .= "</select></td>";
                    }
                    break;
                case "date":
                case "time":
                case "datetime":
                case "timedate":
                case "timestamp":
                    $data = htmlspecialchars($data);
                    $txtVar .= "<td><input type=\"text\" name=\"{$fieldName}\"";
                    if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) {
                        $txtVar .= " style=\"" . $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                    } else {
                        $txtVar .= (isset($value['fieldStyles']) ? " style=\"" . $value['fieldStyles'] . "\"" : "");
                    }
                    if (isset($value['attributes'])) {
                        if (is_callable($value['attributes'])) {
                            $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                            $value['attributes'] = $tmp;
                        } else {
                            if (gettype($value['attributes'] == "array")) {
                                $tmp = "";
                                foreach ($value['attributes'] as $aK => $aV) $tmp .= "{$aK}=\"{$aV}\"";
                                $tmp = substr($tmp, 0, -1);
                                $value['attributes'] = $tmp;
                            }
                        }
                        $txtVar .= " " . $value['attributes'];
                    }

                    $txtVar .= ($data != "" ? "value=\"{$data}\"" : "") . "/></td>";
                    if ($value["options"]) {
                        $tmp["options"] = $value["options"];
                    }

                    $txtVar .= "<script type=\"text/javascript\">";
                    $txtVar .= "date = new Date();";
                    $txtVar .= "$(\"input[name=" . $fieldName . "]\").val(date.getFullYear() + \"-\" + (date.getMonth() + 1 < 10 ? \"0\" : \"\") + (date.getMonth() + 1) + \"-\" + (date.getDate() < 10 ? \"0\" : \"\") + date.getDate());";
                    if ($data) {
                        $txtVar .= "$(\"input[name=" . $fieldName . "]\").val(\"" . $data . "\");";
                    }
                    $txtVar .= "$(\"input[name=" . $fieldName . "]\").datepicker({dateFormat: \"yy-mm-dd\"});";
                    $txtVar .= "</script>";
                    break;
                case "checkbox":
                case "check":
                case "box":
                case "bool":
                case "boolean":
                    $txtVar .= "<td><input type=\"hidden\" value=\"0\" name=\"{$fieldName}\"><input type=\"checkbox\" name=\"{$fieldName}\"";
                    if (isset($value['attributes'])) {
                        if (is_callable($value['attributes'])) {
                            $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                            $value['attributes'] = $tmp;
                        } else {
                            if (gettype($value['attributes'] == "array")) {
                                $tmp = "";
                                foreach ($value['attributes'] as $aK => $aV) $tmp .= "{$aK}=\"{$aV}\"";
                                $tmp = substr($tmp, 0, -1);
                                $value['attributes'] = $tmp;
                            }
                        }
                        $txtVar .= " " . $value['attributes'];
                    }
                    if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) $txtVar .= " style=\"" . $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                    else $txtVar .= (isset($value['fieldStyles']) ? " style=\"" . $value['fieldStyles'] . "\"" : "");
                    $txtVar .= ($data != 0 ? "value=\"1\" checked=\"checked\"" : "value=\"0\"") . " onchange=\"this.value = this.checked ? 1 : 0\"/></td>";
                    break;
                case "multiselectfromlist":
                case "multiselectfromtable":
                    if ($value['multiValues'] == "string") {
                        $data = preg_replace("/" .$value['multiValuesSeparator'] . "{2,}/", $value['multiValuesSeparator'], $data);
                        $data = preg_replace("/(?:^" . $value['multiValuesSeparator'] . "+|" . $value['multiValuesSeparator'] . "+$)/", "", $data);
                        $dataN = explode($value['multiValuesSeparator'], $data);
                    }
                    $txtVar .= "<td><div id=\"selecty\">";
                    foreach ($dataN as $datK => $datV) {
                        $txtVar .= "<select class=\"multi cc\" name=\"{$fieldName}[]\"";
                        if (isset($value['attributes'])) {
                            if (is_callable($value['attributes'])) {
                                $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                                $value['attributes'] = $tmp;
                            } else {
                                if (gettype($value['attributes'] == "array")) {
                                    $tmp = "";
                                    foreach ($value['attributes'] as $aK => $aV) $tmp .= "{$aK}=\"{$aV}\"";
                                    $tmp = substr($tmp, 0, -1);
                                    $value['attributes'] = $tmp;
                                }
                            }
                            $txtVar .= " " . $value['attributes'];
                        }
                        if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) $txtVar .= " style=\"" . $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                        else $txtVar .= (isset($value['fieldStyles']) ? " style=\"" . $value['fieldStyles'] . "\"" : "");
                        $txtVar .= ">";
                        if (strtolower($value['type']) == "multiselectfromlist") {
                            if (isset($value['from']['firstEmpty'])) {
                                $txtVar .= "<option value=\"\"" . ($datV == $extK ? " selected=\"selected\"" : "") . ">...</option>";
                            }
                            $exData = $value['from']['data'];
                        } else {
                            $exData = isset($extData[$value['valueFromTable'] . "_$lang"]) ? $extData[$value['valueFromTable'] . "_$lang"] : $extData[$value['valueFromTable']];
                            if (stripos($options['options'], "enableGrouping")) $exData = Panel::renderTree($exData, $options);
                        }
                        foreach ($exData as $extK => $extV) {
                            $txtVar .= "<option value=\"$extK\"" . ($datV == $extK ? " selected=\"selected\"" : "") . ">$extV</option>";
                        }
                        $txtVar .= "</select><input type=\"button\" value=\"X\" class=\"kat\" onclick=\"deleteSelectList(this, '{$fieldName}')\">";
                    }
                    $txtVar .= "</div><input type=\"button\" value=\"{$value['addButtonDisplayText']}\" id=\"addButton\" onclick=\"addSelectList(this, '{$fieldName}')\"><input type=\"hidden\" name=\"{$fieldName}\" value=\"" . $data . "\"/></td>";
                    break;
                case "multiselectfromtable+fields":
                case "multiselectfromtablewithfields":
                case "multiselectfromtableandfields":
                    $data = htmlspecialchars($data);
                    if ($value['multiValues'] == "string") {
                        $data = preg_replace("/" .$value['multiValuesSeparator'] . "{2,}/", $value['multiValuesSeparator'], $data);
                        $data = preg_replace("/(?:^" . $value['multiValuesSeparator'] . "+|" . $value['multiValuesSeparator'] . "+$)/", "", $data);
                        $dataN = explode($value['multiValuesSeparator'], $data);
                    }
                    if (isset($value['saveColumn'])) {
                        $attrData = $extData[$value['saveColumn'] . ($lang ? "_$lang" : "")];
                        $attrMatch = [];
                        // {%(.+?)(?=%)%}{#(.+?)(?=#)#}.+?{\/#\2#}{\/%\1%}
                        preg_match_all("/{%(.+?)(?=%)%}(.*){%\/\g1%}/", $attrData, $attrMatch);
                        if (isset($attrMatch[1])) {
                            $attrIDs = $attrMatch[1];
                            $attrsTmp = $attrMatch[0];
                            $attrs = [];
                            foreach ($attrsTmp as $aKey => $aValue) {
                                $attrs[$aKey] = [];
                                $tmp1 = [];
                                preg_match_all("/{#(.+?)(?=#)#}(.*){#\/\g1#}/", $aValue, $tmp1);
                                if (isset($tmp1[1])) {
                                    foreach ($tmp1[1] as $tKey => $tValue) {
                                        $attrs[$aKey][$tValue] = $tmp1[2][$tKey];
                                    }
                                }
                            }
                        }
                    } else $attrData = [];
                    $txtVar .= "<td><div id=\"selecty2\">";
                    $fAtrCnt = 0;
                    foreach ($dataN as $datK => $datV) {
                        $txtVar .= "<select class=\"multi2 cc2 atr\" name=\"{$fieldName}[]\" style=\"float: left; ";
                        if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) $txtVar .= $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                        else $txtVar .= (isset($value['fieldStyles']) ? $value['fieldStyles'] : "");
                        if (isset($value['attributes'])) {
                            if (is_callable($value['attributes'])) {
                                $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                                $value['attributes'] = $tmp;
                            } else {
                                if (gettype($value['attributes'] == "array")) {
                                    $tmp = "";
                                    foreach ($value['attributes'] as $aK => $aV) $tmp .= "{$aK}=\"{$aV}\"";
                                    $tmp = substr($tmp, 0, -1);
                                    $value['attributes'] = $tmp;
                                }
                            }
                            $txtVar .= " " . $value['attributes'];
                        }
                        $txtVar .= "\">";
                        $exData = isset($extData[$value['valueFromTable'] . "_$lang"]) ? $extData[$value['valueFromTable'] . "_$lang"] : $extData[$value['valueFromTable']];
                        foreach ($exData as $extK => $extV) {
                            $txtVar .= "<option value=\"$extK\"" . ($datV == $extK ? " selected=\"selected\"" : "") . ">$extV</option>";
                        }
                        $txtVar .= "</select><input type=\"button\" value=\"X\" class=\"kat\" onclick=\"deleteSelectAttrList(this, '{$fieldName}')\">";

                        if (isset($value['fields'])) {
                            foreach ($value['fields'] as $fKey => $fValue) {
                                if ($fValue['type'] == "file" || $fValue['type'] == "image") $fType = "text";
                                else $fType = $fValue['type'];
                                switch (mb_strtolower($fType)) {
                                    case "long_text":
                                    case "textarea":
                                    case "html":
                                    case "tinymce":
                                        $tinymce = strtolower($fType) == "html" || strtolower($fType) == "tinymce" ? " class=\"yes\"" : "";
                                        $txtVar .= "<textarea{$tinymce} name=\"{$fieldName}\"";
                                        if (isset($fValue['styles']) && is_callable($fValue['styles'])) {
                                            $txtVar .= " style=\"" . $fValue['styles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                                        } else {
                                            $txtVar .= (isset($fValue['styles']) ? " style=\"" . $fValue['styles'] . "\"" : "");
                                        }

                                        $txtVar .= ">" . (isset($attrs[$fAtrCnt][$fKey]) ? htmlspecialchars($attrs[$fAtrCnt][$fKey]) : "") . "</textarea>";
                                        break;
                                    default:
                                        $txtVar .= "<input class=\"multi2\"" . (($fValue['type'] == "file" || $fValue['type'] == "image") ? " id=\"fileAtr" . ($lang ? "_$lang" : "") . "_{$fAtrCnt}\"" : "") . " name=\"" . $fKey . "\" type=\"" . $fType . "\"";
                                        if (isset($fValue['styles']) && is_callable($fValue['styles'])) $txtVar .= " style=\"" . $fValue['styles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                                        else $txtVar .= (isset($fValue['styles']) ? " style=\"" . $fValue['styles'] . "\"" : "");
                                        $txtVar .= " value=\"" . (isset($attrs[$fAtrCnt][$fKey]) ? htmlspecialchars($attrs[$fAtrCnt][$fKey]) : "") . "\" />";
                                        break;
                                }
                                if ($fValue['type'] == "file" || $fValue['type'] == "image") {
                                    $txtVar .= "<a href=\"filemanager/dialog.php?type=1&subfolder=&editor=mce_0&field_id=fileAtr" . ($lang ? "_$lang" : "") . "_{$fAtrCnt}\" class=\"fancy\" style=\"float: left; margin-left: 10px; padding-top: 10px\">";
                                    if (isset($value['buttonText']) && is_callable($value['buttonText'])) {
                                        $txtVar .= $value['buttonText'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
                                    } else {
                                        $txtVar .= isset($value['buttonText']) ? $value['buttonText'] : "Wybierz";
                                    }

                                    $txtVar .= "</a>";
                                }
                                $fAtrCnt++;
                            }
                        }
                        // $txtVar .= "<input id=\"wartosc1\" value=\"bawełna lub poliester\" class=\"multi2\" type=\"text\" style=\"width: 260px; line-height: 17px; float: left; margin-left: 10px;\" name=\"atr1\">";

                    }
                    $txtVar .= "</div><input type=\"button\" value=\"{$value['addButtonDisplayText']}\" id=\"addButton2\" onclick=\"addSelectAttrList(this, '{$fieldName}')\"><input type=\"hidden\" name=\"{$fieldName}\" value=\"" . $data . "\"/><input type=\"hidden\" name=\"{$value['saveColumn']}" . ($lang ? "_$lang" : "") . "\" value=\"" . $data . "\"/></td>";
                    break;
                case "filelist":
                    if ($value['path'] && file_exists($value['path'])) {
                        $txtVar .= "<td><select name=\"{$fieldName}\"";
                        if (isset($value['attributes'])) {
                            if (is_callable($value['attributes'])) {
                                $tmp .= $value['attributes'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]);
                                $value['attributes'] = $tmp;
                            } else {
                                if (gettype($value['attributes'] == "array")) {
                                    $tmp = "";
                                    foreach ($value['attributes'] as $aK => $aV) $tmp .= "{$aK}=\"{$aV}\"";
                                    $tmp = substr($tmp, 0, -1);
                                    $value['attributes'] = $tmp;
                                }
                            }
                            $txtVar .= " " . $value['attributes'];
                        }
                        if (isset($value['fieldStyles']) && is_callable($value['fieldStyles'])) {
                            $txtVar .= " style=\"" . $value['fieldStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                        } else {
                            $txtVar .= (isset($value['fieldStyles']) ? " style=\"" . $value['fieldStyles'] . "\"" : "") . ">";
                        }
                        if (isset($value['firstEmpty']) && $value['firstEmpty'] != false) $txtVar .= "<option value=\"\"" . ($sValue[$value['value']] == $data ? " selected=\"selected\"" : "") . ">" . ($value['firstEmptyDisplayText'] ? $value['firstEmptyDisplayText'] : "...") . "</option>";
                        $opt = ["include" => isset($value["include"]) ? $value["include"] : "/.php$/i"];
                        if (isset($value["exclude"])) $opt = array_merge($opt, ["exclude" => $value["exclude"]]);
                        $files = Utils::dirFiles($value['path'], $opt);
                        foreach ($files as $sKey => $sValue) {
                            if (isset($value['fileNameOnly']) && $value['fileNameOnly'] == true) $sValue = substr("/" . $sValue, strrpos("/" . $sValue, "/") + 1);
                            $txtVar .= "<option value=\"" . $sValue . "\"" . ($sValue == $data ? " selected=\"selected\"" : "") . ">" . $sValue . "</option>";
                        }
                        $txtVar .= "</select></td>";
                    }
                    break;
                default:
                    break;
            }
        }
        $fileCounter = 1;
        $externalData = false;
        $post = [];
        if ($id != null) {
            $post['id'] = $id;
        }

        $options = ['options' => ""];

        foreach ($editorFields as $key => $value) {
            if (gettype($value) != "array") {
                $options[$key] = $value;
                unset($editorFields[$key]);
                continue;
            }
        }

        $options['options'] = " " . $options['options'];

        $noDisplay = "";
        if (count($this->langs) == 1) $noDisplay = " style=\"display: none\"";

        $editor = "<form style=\"width: 100%\" action=\"content.php?k={$this->k}&o=" . ($id == null ? 1 : 2) . "\" name=\"wczytaj\" enctype=\"multipart/form-data\" method=\"POST\">
        <div id=\"tabContaier\">
            <ul{$noDisplay}>";
        foreach ($this->langs as $value) {
            $editor .= "<li><a" . ($this->defaultLang == $value ? " class=\"active\"" : "") . " href=\"#tab_{$value}\">" . strtoupper($value) . "</a></li>";
        }
        $editor .= "</ul>
        <div class=\"tabDetails\">";

        if ($id != null) {
            $entry = $this->table->getAllLang($id);
        }
        // GETTING TABLES WITH DATA
        $dataOld = [];
        $limits = [];
        $separators = [];
        $uniques = [];
        foreach ($editorFields as $dKey => $dValue) {
            if (isset($dValue['valueFromTable']) || strtolower($dValue['type']) == "multiselectfromlist") {
                if (strtolower($dValue['type']) == "multiselectfromlist") $dValue['valueFromTable'] = $dKey;
                $separators[$dValue['valueFromTable']] = $dValue['multiValuesSeparator'] ? $dValue['multiValuesSeparator'] : ",";
                if (isset($dValue['limit'])) {
                    $limits[$dValue['valueFromTable']] = ['limit' => $dValue['limit']];
                }
                if (isset($dValue['unique'])) {
                    $uniques[$dValue['valueFromTable']] = ['unique' => $dValue['unique']];
                }
                if (strtolower($dValue['lang']) != "all") {
                    foreach ($this->langs as $lang) {
                        $separators[$dValue['valueFromTable'] . "_$lang"] = $dValue['multiValuesSeparator'] ? $dValue['multiValuesSeparator'] : ",";
                        if (isset($dValue['limit'])) {
                            $limits[$dValue['valueFromTable'] . "_$lang"] = ['limit' => $dValue['limit']];
                        }
                        if (isset($dValue['unique'])) {
                            $uniques[$dValue['valueFromTable'] . "_$lang"] = ['unique' => $dValue['unique']];
                        }
                    }
                }
                if (!isset($dataOld[$dValue['valueFromTable']])) {
                    $externalData = true;
                    $arr = [];
                    if (isset($dValue['valueFromField'])) array_push($arr, $dValue['valueFromField']);
                    if (isset($dValue['valueDisplayTextFromField'])) array_push($arr, $dValue['valueDisplayTextFromField']);
                    if (stripos($options['options'], "enableGrouping") && strtolower($dValue['type']) == "multiselectfromtable") {
                        array_push($arr, $options['groupParentColumn']);
                    }
                    if (strtolower($dValue['lang']) != "all") {
                        foreach ($this->langs as $lang) {
                            if ($lang == $this->defaultLang) $nLang = "";
                            else $nLang = $lang;
                            $dataOld[$dValue['valueFromTable'] . "_$lang"] = [$dValue['valueFromField'], $dValue['valueDisplayTextFromField']];
                            if (strtolower($dValue['type']) == "multiselectfromlist") {
                                $dataOld[$dValue['valueFromTable'] . "_$lang"][2] = $this->table->db->select($dValue['valueFromTable'] . $nLang, $arr);
                            } else {
                                $dataOld[$dValue['valueFromTable'] . "_$lang"][2] = [];
                                if ($dValue['from'] == null) $dValue['from'] = ['data' => []];
                                foreach ($dValue['from']['data'] as $dVKey => $dVVal) {
                                    array_push($dataOld[$dValue['valueFromTable'] . "_$lang"][2], [$dataOld[$dValue['valueFromTable'] . "_$lang"][0] => $dVKey, $dataOld[$dValue['valueFromTable'] . "_$lang"][1] => $dVVal]);
                                }
                            }
                            if (stripos($options['options'], "enableGrouping") && strtolower($dValue['type']) == "multiselectfromtable") {
                                $dataOld[$dValue['valueFromTable'] . "_$lang"][2] = Panel::renderTree($dataOld[$dValue['valueFromTable'] . "_$lang"][2], $options);
                            }
                            //$dataOld[$dValue['valueFromTable'] . "_$lang"][2] = $this->table->db->select($dValue['valueFromTable'] . $nLang, (isset($dValue['valueFromField']) && isset($dValue['valueDisplayTextFromField']) ? [$dValue['valueFromField'], $dValue['valueDisplayTextFromField']] : "*"));
                            if (isset($entry[$lang][$dValue['saveColumn']])) $dataOld[$dValue['valueFromTable'] . "_$lang"][3] = [$dValue['saveColumn'] . ($lang ? "_$lang" : ""), $entry[$lang][$dValue['saveColumn']]];
                        }
                    } else {
                        $dataOld[$dValue['valueFromTable']] = [$dValue['valueFromField'], $dValue['valueDisplayTextFromField']];
                        if (strtolower($dValue['type']) == "multiselectfromlist") {
                            $dataOld[$dValue['valueFromTable']][2] = [];
                            foreach ($dValue['from']['data'] as $dVKey => $dVVal) {
                                array_push($dataOld[$dValue['valueFromTable']][2], [$dataOld[$dValue['valueFromTable']][0] => $dVKey, $dataOld[$dValue['valueFromTable']][1] => $dVVal]);
                            }
                        } else $dataOld[$dValue['valueFromTable']][2] = $this->table->db->select($dValue['valueFromTable'], $arr);
                        if (stripos($options['options'], "enableGrouping") && strtolower($dValue['type']) == "multiselectfromtable") {
                            $dataOld[$dValue['valueFromTable']][2] = Panel::renderTree($dataOld[$dValue['valueFromTable']][2], $options);
                        }
                        //$dataOld[$dValue['valueFromTable']][2] = $this->table->db->select($dValue['valueFromTable'], (isset($dValue['valueFromField']) && isset($dValue['valueDisplayTextFromField']) ? [$dValue['valueFromField'], $dValue['valueDisplayTextFromField']] : "*"));
                            if (isset($entry[$this->defaultLang][$dValue['saveColumn']])) $dataOld[$dValue['valueFromTable']][3] = [$dValue['saveColumn'], $entry[$this->defaultLang][$dValue['saveColumn']]];
                    }
                }
            }
        }
        $data = [];
        foreach ($dataOld as $dKey => $dValue) {
            $data[$dKey] = [];
            foreach ($dValue[2] as $key => $value) {
                $data[$dKey][$value[$dValue[0]]] = $value[$dValue[1]];
                if (stripos($options['options'], "enableGrouping") && isset($value['txt'])) {
                    $data[$dKey][$value[$dValue[0]]] = $value['txt'];
                    unset($value['txt']);
                }
            }
            if (isset($dValue[3])) $data[$dValue[3][0]] = $dValue[3][1];
        }

        // LANGUAGE DEPENDENT
        foreach ($this->langs as $lang) {
            $editor .= "<div id=\"tab_{$lang}\" class=\"tabContents\">";
            foreach ($editorFields as $key => $value) {
                if (!isset($value['lang'])) $value['lang'] = "all";
                if (strtoupper($value['lang']) == "ALL") {
                    continue;
                }

                if (isset($value['fieldName'])) {
                    if (is_callable($value['fieldName'])) {
                        $fieldName = $value['fieldName'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
                    } else {
                        $fieldName = $value['fieldName'];
                    }

                } else {
                    $fieldName = $key . "_" . $lang;
                }
                $value['fieldName'] = $fieldName;
                $post[$fieldName] = ['lang' => $lang, 'column' => $key];
                if ($value['saveColumn']) $post["{$value['saveColumn']}_{$lang}"] = ['lang' => $lang, 'column' => $value['saveColumn']];
                $editor .= "<table class=\"add_table\">";
                if (isset($value['value']) && is_callable($value['value'])) {
                    $value['value'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
                } else {
                    $editor .= "<tr><td";
                    if (isset($value['displayStyles']) && is_callable($value['displayStyles'])) {
                        $editor .= " style=\"" . $value['displayStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                    } else {
                        $editor .= (isset($value['displayStyles']) ? " style=\"" . $value['displayStyles'] . "\"" : "");
                    }

                    $editor .= ">";
                    if (isset($value['displayName']) && is_callable($value['displayName'])) {
                        $editor .= $value['displayName'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
                    } else {
                        $editor .= isset($value['displayName']) ? $value['displayName'] : $key;
                    }

                    $editor .= "</td>";
                    if (strtolower($value['type']) == "multiselectfromlist" || strtolower($value['type']) == "multiselectfromtable") $this->flags["editor"]["listed"] = true;
                    if (preg_match("/multiselectfromtable.+/i", $value['type']) > 0) $this->flags["editor"]["listedAtr"] = true;
                    type($editor, ["lang" => $lang, "defaultLang" => $this->defaultLang], $key, $value, $entry[$lang][$key], $fileCounter, $post, isset($value['valueFromTable']) ? $data : NULL);
                    $editor .= "</tr>";
                }
                $editor .= "</table>";
            }
            $editor .= "</div>";
        }
        $editor .= "</div><hr style=\"width: 100%; border: none; border-bottom: 1px solid lightgray; height: 10px\" />";

        // LANGUAGE INDEPENDENT
        foreach ($editorFields as $key => $value) {
            if (strtoupper($value['lang']) != "ALL") {
                continue;
            }

            $lang = $this->defaultLang;
            if (isset($value['fieldName'])) {
                if (is_callable($value['fieldName'])) {
                    $fieldName = $value['fieldName'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
                } else {
                    $fieldName = $value['fieldName'];
                }

            } else {
                $fieldName = $key;
            }
            $value['fieldName'] = $fieldName;
            $post[$fieldName] = ['column' => $key];
            if ($value['saveColumn']) $post[$value['saveColumn']] = ['column' => $value['saveColumn']];
            $editor .= "<table class=\"add_table\">";
            if (isset($value['value']) && is_callable($value['value'])) {
                $value['value'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
            } else {
                $editor .= "<tr><td";
                if (isset($value['displayStyles']) && is_callable($value['displayStyles'])) {
                    $editor .= " style=\"" . $value['displayStyles'](["db" => $this->table->db, "table" => $this->table, "key" => $entryKey, "value" => $entryValue]) . "\"";
                } else {
                    $editor .= (isset($value['displayStyles']) ? " style=\"" . $value['displayStyles'] . "\"" : "");
                }

                $editor .= ">";
                if (isset($value['displayName']) && is_callable($value['displayName'])) {
                    $editor .= $value['displayName'](["db" => $this->table->db, "table" => $this->table, "id" => $id, "key" => $key, "value" => $value], $renderData);
                } else {
                    $editor .= isset($value['displayName']) ? $value['displayName'] : $key;
                }

                $editor .= "</td>";
                if (strtolower($value['type']) == "multiselectfromlist" || strtolower($value['type']) == "multiselectfromtable") $this->flags["editor"]["listed"] = true;
                if (preg_match("/multiselectfromtable.+/i", $value['type']) > 0) $this->flags["editor"]["listedAtr"] = true;
                type($editor, ["lang" => null, "defaultLang" => $this->defaultLang], $key, $value, $entry[$lang][$key], $fileCounter, $post, isset($value['valueFromTable']) ? $data : NULL);
                $editor .= "</tr>";
            }
            $editor .= "</table>";
        }

        if (stripos($options['options'], "enableLeveling")) {
            // PARENT EDIT
            $editor .= "<table class=\"add_table\">";
            
            $editor .= "</table>";
        }

        // COMMENTS SECTION

        if (stripos($options['options'], "enableComments")) {
            if (!isset($options['commentsTable'])) $options['commentsTable'] = "komentarze";
            if (!isset($options['commentsIDColumn'])) $options['commentsIDColumn'] = "id";
            if (!isset($options['commentsArticleIDColumn'])) $options['commentsArticleIDColumn'] = "artykul_id";
            if (!isset($options['commentsAuthorColumn'])) $options['commentsAuthorColumn'] = "autor";
            if (!isset($options['commentsTextColumn'])) $options['commentsTextColumn'] = "tresc";
            if (!isset($options['commentsDateColumn'])) $options['commentsDateColumn'] = "data";
            if (!isset($options['commentsEditedColumn'])) $options['commentsEditedColumn'] = "edytowany";
            if (!isset($options['commentsReasonColumn'])) $options['commentsReasonColumn'] = "powod";
            if (!isset($options['commentsShowColumn'])) $options['commentsShowColumn'] = "pokaz";

            if (!$this->table->db->tableExists($options['commentsTable'])) {
                $options['options'] = preg_replace("/enableComments/i", "", $options['options']);
            }
        }

        if (stripos($options['options'], "enableComments")) {
            $test = ["commentsIDColumn", "commentsArticleIDColumn", "commentsAuthorColumn", "commentsTextColumn", "commentsDateColumn", "commentsEditedColumn", "commentsReasonColumn", "commentsShowColumn"];
            $comments = new Module($this->table->db, LANGS, $options['commentsTable'], $this->table->ref);
            $testCols = $comments->getColumnNames();
            foreach ($test as $value) {
                if (!in_array($options[$value], $testCols)) { // COLUMN NOT FOUND, SO DISABLE LEVELING
                    $options['options'] = preg_replace("/enableComments/i", "", $options['options']);
                    break;
                }
            }
        }

        if (stripos($options['options'], "enableComments")) {
            $editor .= "<hr style=\"width: 100%; border: none; border-bottom: 1px solid lightgray; height: 10px\" /><h1 style=\"font-size: 28px; font-weight: bold; font-family: 'Exo 2', 'Geneva', serif; padding: 25px 0 45px 0\">Komentarze</h1><div>";
            $commentsList = $comments->getWhere(null, "*");
            /*
            echo "<pre>";
            //echo $_SERVER['REMOTE_ADDR'] . " " . $_SERVER['HTTP_X_FORWARDED_FOR'];
            var_dump($commentsList);
            echo "</pre>";
            */
            foreach ($commentsList as $cKey => $cValue) {
                $user = $cValue[$options['commentsAuthorColumn']] ? htmlspecialchars($cValue[$options['commentsAuthorColumn']]) : "&lt;anonim&gt;";
                $commentText = $cValue[$options['commentsTextColumn']] ? htmlspecialchars($cValue[$options['commentsTextColumn']]) : "";
                if ($commentText != "") {
                    $editor .= "<div class=\"comment\"><p>W dniu <span class=\"commentDate\">{$cValue[$options['commentsDateColumn']]}</span> użytkownik <span class=\"commentAuthor\">{$user}</span> pisze:";
                    $editor .= "<span class=\"commentShow\"><label>Pokazuj komentarz</label><input type=\"checkbox\" name=\"commentShow\"" . ($cValue[$options['commentsShowColumn']] != 0 ? " checked=\"checked\"" : "") . " /></span></p>";
                    $editor .= "<p class=\"commentText\">{$cValue[$options['commentsTextColumn']]}</p></div>";
                }
            }
            $editor .= "</div>";
        }

        $editor .= "<table class=\"add_table\">
            <tr><td style=\"text-align: right;\">
               <input style=\"width: 120px; height: 40px; \" type=\"submit\" class=\"anuluj\"  name=\"anuluj\"  value=\"Anuluj zmiany\">
               <input style=\"width: 120px; height: 40px; \" type=\"submit\" class=\"zapisz\"  name=\"submit\" value=\"" . ($id == null ? "Dodaj" : "Zapisz zmiany") . "\">
            </td></tr>
        </table></div></form>";
        echo $editor;
        foreach ($data as $dKey => $dValue) {
            if (gettype($dValue) == "array") {
                $data2 = [];
                foreach ($dValue as $ddKey => $ddValue) {
                    array_push($data2, [$ddValue, $ddKey]);
                }
                $data[$dKey] = $data2;
            }
        }
        if ($this->flags["editor"]["listed"] || $this->flags["editor"]["listedAtr"]) { ?>
        <script type="text/javascript">
            tables = <?php echo json_encode($data); ?>;
            limits = <?php echo json_encode($limits); ?>;
            separators = <?php echo json_encode($separators); ?>;
            uniques = <?php echo json_encode($uniques); ?>;
            values = [];<?php
            if ($this->flags["editor"]["listed"] == true) { ?>
            $("select.multi.cc").each(function() {
                changeSelectList(this.getAttribute("name").replace(new RegExp("[\[\]]", "g"), ""));
            });
            focused = function(t) {
                tableName = this.getAttribute("name").replace(new RegExp("[\[\]]", "g"), "");
                if (!values[tableName]) values[tableName] = [];
                values[tableName][$(this).index()] = this.value;
                changeSelectList(tableName);
            };
            $("div#selecty").on("click focus", "select.multi.cc", focused).change(function(e) {
                select = $("select.multi.cc[name=\"" + tableName + "[]\"]", this);
                if (uniques[tableName]) {
                    alreadyUsed = false;
                    select.each(function() {
                        if (this != e.target) {
                            if (this.value == e.target.value) {
                                alreadyUsed = true;
                                return false;
                            }
                        }
                    });
                    if (alreadyUsed) {
                        e.target.value = values[tableName][$(e.target, this).index()];
                        alert("Kategoria już została wybrana.");
                        return false;
                    }
                }
                values[tableName][$(this).index()] = this.value;
                changeSelectList(tableName);
            });
            function addSelectList(jQ, table) {
                var select = $("select.multi.cc[name=\"" + table + "[]\"]");
                var options = select.eq(0).find("option");
                if (select.length >= options.length) {
                    alert("Osiągnięto limit wybranych kategorii.");
                    return false;
                }
                if ((limits[table] && select.length >= (Array.isArray(limits[table].limit) ? limits[table].limit[1] : limits[table].limit)) || select.length >= tables[table].length) {
                    alert("Osiągnięto limit wybranych kategorii.");
                    return false;
                }
                
                selectHTML = "<select class=\"multi cc\" name=\"" + table + "[]\">";
                for (i in tables[table]) {
                    selectHTML += "<option value=\"" + tables[table][i][1] + "\">" + tables[table][i][0] + "</option>";
                }
                selectHTML += "<input type=\"button\" value=\"X\" class=\"kat\" onclick=\"deleteSelectList(this, '" + table + "')\">";
                /*
                var selectHTML = $("<select class=\"multi cc\" name=\"" + table + "[]\"></select>");
                selectHTML.append(options.clone());
                var deleteButton = $("<input type=\"button\" value=\"X\" class=\"kat\" onclick=\"deleteSelectList(this, '" + table + "')\">");
                var tmp = selectHTML; */
                // $(jQ).prev().append(tmp).append(deleteButton);
                var tmp = $(selectHTML);
                $(jQ).prev().append(tmp);
                tmp[0].selectedIndex = -1;
                changeSelectList(table);
            }
            function deleteSelectList(jQ, table) {
                select = $("select.multi.cc[name=\"" + table + "[]\"]");
                min = limits[table] && Array.isArray(limits[table].limit) ? limits[table].limit[1] : 1;
                if (select.length <= min) {
                    alert("Minimum pozycji: " + min);
                    return false;
                }
                $(jQ).prev().remove();
                $(jQ).remove();
                changeSelectList(table);
            }
            function changeSelectList(table) {
                val = [];
                select = $("select.multi.cc[name=\"" + table + "[]\"]");
                select.each(function() {
                    val.push(this.value);
                });
                val = val.join(separators[table]).replace(new RegExp(separators[table] + "{2,}", "g"), separators[table]).replace(new RegExp("^" + separators[table] + "|" + separators[table] + "$", "g"), "");
                select.parents("td:first").find("input[type=hidden][name=" + table + "]").val(val);
            } <?php
            }
            if ($this->flags["editor"]["listedAtr"] == true) { ?>
            $("select.multi2.cc2").each(function() {
                changeSelectAttrList(this.getAttribute("name").replace(new RegExp("[\[\]]", "g"), ""));
            });
            function changeCall(jQ) {
                if (!jQ) jQ = this; else jQ = jQ.target;
                if (!$(jQ).is("select")) jQ = $(jQ).prevAll("select:first"); else jQ = $(jQ);
                changeSelectAttrList(jQ.attr("name").replace(new RegExp("[\[\]]+", "g"), ""));
            }
            $("div#selecty2").on("click focus blur", "select.multi2.cc2", changeCall).change(changeCall);
            $("input#addButton2").each(function() {
                $(this).parents("td:first").on("click focus blur keyup change", "input:not([type=button]), a", changeCall);
            });
            $("a[href*=file].fancy").on("click", function() {
                href = this.href.match(new RegExp("field_id=(.+).*$", "i"));
                if (!href) {
                    $(this).off("click");
                    return true;
                }
                href = href[1];
                input = $(this).parent().find("input[id=" + href+"]");
                setTimeout(function() {
                    timer = setInterval(function() {
                        if ($("iframe[id*=fancy]").length < 1) {
                            input.change();
                            clearTimeout(timer);
                        }
                    }, 1000);
                }, 1500);
            });
            function enumerateSelectAttrList(table) {
                select = $("select.multi2.cc2[name=\"" + table + "[]\"]");
                counter = 0;
                max = select.length;
                select.each(function() {
                    row = $(this).first().nextUntil("select, div").addBack();
                    row.each(function() {
                        if (!$(this).is("a[href], input[id]")) return true;
                        if ($(this).is("a[href]")) {
                            this.href = this.href.replace(new RegExp("(fileAtr_[^_]*_)([0-9]+)", "g"), "$1" + counter);
                        }
                        if ($(this).is("input[id]")) {
                            this.id = this.id.replace(new RegExp("(fileAtr_[^_]*_)([0-9]+)", "g"), "$1" + counter);
                        }
                        if ($(this).is("a[href]") && ($(this).prev().is("input[id]") && $(this).next().is("input[id]"))) counter++;
                    });
                    counter++;
                });
            }
            function changeSelectAttrList(table) {
                val = [];
                v = [];
                select = $("select.multi2.cc2[name=\"" + table + "[]\"]");
                select.each(function() {
                    if (typeof uniques[table] != "undefined" && uniques[table].unique == true && v.indexOf(this.value) > -1) {
                        this.value = "";
                    } else {
                        if (this.value != "") {
                            v.push(this.value);
                            row = $(this).nextUntil("select, div", "input[name]");
                            val2 = {id: this.value};
                            row.each(function() {
                                val2[this.name] = this.value;
                            });
                            val.push(val2);
                        }
                    }
                });
                console.log(v, val);
                $("input[name=" + table + "]").val(v.join(separators[table]));
                str = "";
                for (i in val) {
                    str += "{%" + i + "%}";
                    for (j in val[i]) {
                        if (j.toLowerCase() == "id") continue;
                        str += "{#" + j + "#}" + val[i][j] + "{#/" + j + "#}";
                    }
                    str += "{%/" + i + "%}";
                }
                $("input[name=" + table + "]").nextAll("input[type=hidden]:not([name=" + table + "]):first").val(str);
            }
            function addSelectAttrList(jQ, table) {
                select = $("select.multi2.cc2[name=\"" + table + "[]\"]");
                if ((limits[table] && select.length >= limits[table].limit)) { // || select.length >= tables[table].length
                    alert("Osiągnięto limit wybranych kategorii.");
                    return false;
                }
                row = select.first().nextUntil("select, div").addBack().clone();
                row.each(function() {
                    if (!$(this).is("input[name]")) return true;
                    this.value = "";
                });
                select.parent().append(row);
                enumerateSelectAttrList(table);
                changeSelectAttrList(table);
            }
            function deleteSelectAttrList(jQ, table) {
                select = $("select.multi2.cc2[name=\"" + table + "[]\"]");
                min = limits[table] && Array.isArray(limits[table].limit) ? limits[table].limit[1] : 1;
                if (select.length <= min) {
                    alert("Minimum pozycji: " + min);
                    return false;
                }
                select = $(jQ).prev().add($(jQ).nextUntil("select, div").addBack());
                $(select).remove();
                enumerateSelectAttrList(table);
                changeSelectAttrList(table);
            }
            <?php
            } ?>
        </script><?php
        }
        $_SESSION['POST'] = $post;
    }
}
